/*  x11amp - graphically mp3 player..
 *  Copyright (C) 1998-1999  Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>
#include "x11ampctrl.h"
#include "../x11amp/controlsocket.h"

static gpointer remote_read_packet(gint fd,ServerPktHeader *pkt_hdr)
{
	gpointer data = NULL;
	
	if(read(fd,pkt_hdr,sizeof(ServerPktHeader)) == sizeof(ServerPktHeader))
	{
		if(pkt_hdr->data_length)
		{
			data = g_malloc0(pkt_hdr->data_length);
			read(fd,data,pkt_hdr->data_length);
		}
	}
	return data;
}

static void remote_read_ack(gint fd)
{
	gpointer data;
	ServerPktHeader pkt_hdr;
	
	
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
		g_free(data);
	
}

static void remote_send_packet(gint fd,guint32 command,gpointer data,guint32 data_length)
{
	ClientPktHeader pkt_hdr;

	pkt_hdr.version = X11AMP_PROTOCOL_VERSION;
	pkt_hdr.command = command;
	pkt_hdr.data_length = data_length;
	write(fd,&pkt_hdr,sizeof(ClientPktHeader));
	if(data_length && data)
		write(fd,data,data_length);
}

static void remote_send_guint32(gint session,guint32 cmd,guint32 val)
{
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	remote_send_packet(fd,cmd,&val,sizeof(guint32));
	remote_read_ack(fd);
	close(fd);
}

static void remote_send_boolean(gint session,guint32 cmd,gboolean val)
{
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	remote_send_packet(fd,cmd,&val,sizeof(gboolean));
	remote_read_ack(fd);
	close(fd);
}

static void remote_send_string(gint session,guint32 cmd,gchar *string)
{
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	remote_send_packet(fd,cmd,string,string ? strlen(string) + 1 : 0);
	remote_read_ack(fd);
	close(fd);
}	

static gboolean remote_cmd(gint session,guint32 cmd)
{
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return FALSE;
	remote_send_packet(fd,cmd,NULL,0);
	remote_read_ack(fd);
	close(fd);
	
	return TRUE;
}

static gboolean remote_get_gboolean(gint session,gint cmd)
{
	ServerPktHeader pkt_hdr;
	gboolean ret = FALSE;
	gpointer data;
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return ret;
	remote_send_packet(fd,cmd,NULL,0);
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
	{
		ret = *((gboolean *)data);
		g_free(data);
	}
	remote_read_ack(fd);
	close(fd);
	
	return ret;
}

static guint32 remote_get_gint(gint session,gint cmd)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd,ret = 0;

	if(!(fd=x11amp_connect_to_session(session))) 
		return ret;
	remote_send_packet(fd,cmd,NULL,0);
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
	{
		ret = *((gint *)data);
		g_free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return ret;
}

gchar *remote_get_string(gint session,gint cmd)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return NULL;
	remote_send_packet(fd,cmd,NULL,0);
	data = remote_read_packet(fd,&pkt_hdr);
	remote_read_ack(fd);
	close(fd);
	return data;
}

gchar *remote_get_string_pos(gint session,gint cmd,guint32 pos)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd;

	if(!(fd=x11amp_connect_to_session(session))) 
		return NULL;
	remote_send_packet(fd,cmd,&pos,sizeof(guint32));
	data = remote_read_packet(fd,&pkt_hdr);
	remote_read_ack(fd);
	close(fd);
	return data;
}

gint x11amp_connect_to_session(gint session)
{
	gint fd;
	struct sockaddr_un saddr;
	
	if((fd=socket(AF_UNIX,SOCK_STREAM,0))!=-1)
	{
		saddr.sun_family=AF_UNIX;
	    	sprintf(saddr.sun_path,"%s/x11amp_%s.%d",g_get_tmp_dir(),g_get_user_name(),session);
		if(connect(fd,(struct sockaddr*)&saddr,sizeof(saddr))!=-1)
			return fd;
	}
	close(fd);
	return 0;
}

void x11amp_remote_playlist(gint session,gchar **list,gint num,gboolean enqueue)
{
	gint fd,dummy,i;
	gpointer data,ptr;
	gint data_length;
	guint32 len;

	if(!enqueue) 
		x11amp_remote_playlist_clear(session);

	for(i = 0, data_length = 0;i < num; i++)
		data_length += strlen(list[i]) + 5;
	if(data_length)
	{
		data_length += 4;
		data = g_malloc(data_length);
		for(i = 0, ptr = data;i < num; i++)
		{
			len = strlen(list[i]) + 1;
			*((guint32 *)ptr) = len;
			ptr += 4;
			memcpy(ptr,list[i],len);
			ptr += len;
		}
		*((guint32 *)ptr) = 0;
		if(!(fd=x11amp_connect_to_session(session)))
			return;
		remote_send_packet(fd,CMD_PLAYLIST_ADD,data,data_length);
		remote_read_ack(fd);
		close(fd);
	}

	if(!enqueue) 
		x11amp_remote_play(session);
}



gint x11amp_remote_get_version(gint session)
{
	return remote_get_gint(session,CMD_GET_VERSION);
}

void x11amp_remote_play_files(gint session,GList *list)
{
	x11amp_remote_playlist_clear(session);
	x11amp_remote_add_files(session,list);
	x11amp_remote_play(session);
}

void x11amp_remote_playlist_add(gint session,GList *list)
{
	gchar **str_list;
	GList *node;
	gint i,num;
	
	num = g_list_length(list);
	str_list = g_malloc0(num * sizeof(gchar *));
	for(i = 0,node = list;i < num && node; i++, node = g_list_next(node))
	{
		str_list[i] = node->data;
	}
	x11amp_remote_playlist(session,str_list,num,TRUE);
	g_free(str_list);
}

void x11amp_remote_play(gint session)
{
	remote_cmd(session,CMD_PLAY);
}

void x11amp_remote_pause(gint session)
{
	remote_cmd(session,CMD_PAUSE);
}

void x11amp_remote_stop(gint session)
{
	remote_cmd(session,CMD_STOP);
}

gboolean x11amp_remote_is_playing(gint session)
{
	return remote_get_gboolean(session,CMD_IS_PLAYING);
}

gboolean x11amp_remote_is_paused(gint session)
{
	return remote_get_gboolean(session,CMD_IS_PAUSED);
}

gint x11amp_remote_get_playlist_pos(gint session)
{
	return remote_get_gint(session,CMD_GET_PLAYLIST_POS);
}

void x11amp_remote_set_playlist_pos(gint session,gint pos)
{
	remote_send_guint32(session,CMD_SET_PLAYLIST_POS,pos);
}

gint x11amp_remote_get_playlist_length(gint session)
{
	return remote_get_gint(session,CMD_GET_PLAYLIST_LENGTH);
}

void x11amp_remote_playlist_clear(gint session)
{
	remote_cmd(session,CMD_PLAYLIST_CLEAR);
}

gint x11amp_remote_get_output_time(gint session)
{
	return remote_get_gint(session,CMD_GET_OUTPUT_TIME);
}

void x11amp_remote_jump_to_time(gint session,gint pos)
{
	remote_send_guint32(session,CMD_JUMP_TO_TIME,pos);
}

void x11amp_remote_get_volume(gint session,gint *vl,gint *vr)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;

	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	remote_send_packet(fd,CMD_GET_VOLUME,NULL,0);
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
	{
		*vl = ((guint32 *)data)[0];
		*vr = ((guint32 *)data)[1];
		g_free(data);
	}
	remote_read_ack(fd);
	close(fd);
}

gint x11amp_remote_get_main_volume(gint session)
{
	gint vl,vr;
	
	x11amp_remote_get_volume(session,&vl,&vr);
	
	return (vl > vr) ? vl : vr;
}

gint x11amp_remote_get_balance(gint session)
{
	return remote_get_gint(session,CMD_GET_BALANCE);
}

void x11amp_remote_set_volume(gint session,gint vl,gint vr)
{
	gint fd;
	guint32 v[2];

	if(vl < 0)
		vl = 0;
	if(vl > 100)
		vl = 100;
	if(vr < 0)
		vr = 0;
	if(vr > 100)
		vr = 100;
	
	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	v[0] = vl;
	v[1] = vr;
	remote_send_packet(fd,CMD_SET_VOLUME,v,2 * sizeof(guint32));
	remote_read_ack(fd);
	close(fd);
}

void x11amp_remote_set_main_volume(gint session,gint v)
{
	gint b,vl,vr;
		
	b = x11amp_remote_get_balance(session);
	
	if(b < 0)
	{
		vl = v;
		vr = (v * (100 - abs(b))) / 100;
	}
	else if(b > 0)
	{
		vl = (v * (100 - b)) / 100;
		vr = v;
	}
	else
		vl = vr = v;
	x11amp_remote_set_volume(session, vl,vr);
}

void x11amp_remote_set_balance(gint session,gint b)
{
	gint v,vl,vr;
	
	if(b < -100)
		b = -100;
	if(b > 100)
		b = 100;
	
	v = x11amp_remote_get_main_volume(session);
	
	if(b < 0)
	{
		vl = v;
		vr = (v * (100 - abs(b))) / 100;
	}
	else if(b > 0)
	{
		vl = (v * (100 - b)) / 100;
		vr = v;
	}
	else
		vl = vr = v;
	x11amp_remote_set_volume(session, vl,vr);
}
		
gchar *x11amp_remote_get_skin(gint session)
{
	remote_get_string(session,CMD_GET_SKIN);
}

void x11amp_remote_set_skin(gint session,gchar *skinfile)
{
	remote_send_string(session,CMD_SET_SKIN,skinfile);
}


gchar *x11amp_remote_get_playlist_file(gint session,gint pos)
{
	return remote_get_string_pos(session,CMD_GET_PLAYLIST_FILE,pos);
}

gchar *x11amp_remote_get_playlist_title(gint session,gint pos)
{
	return remote_get_string_pos(session,CMD_GET_PLAYLIST_TITLE,pos);
}

gint x11amp_remote_get_playlist_time(gint session,gint pos)
{
	ServerPktHeader pkt_hdr;
	gpointer data;
	gint fd,ret = 0;
	guint32 p = pos;

	if(!(fd=x11amp_connect_to_session(session))) 
		return ret;
	remote_send_packet(fd,CMD_GET_PLAYLIST_TIME,&p,sizeof(guint32));
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
	{
		ret = *((gint *)data);
		g_free(data);
	}
	remote_read_ack(fd);
	close(fd);
	return ret;
}

void x11amp_remote_get_info(gint session,gint *rate,gint *freq,gint *nch)
{
	ServerPktHeader pkt_hdr;
	gint fd;
	gpointer data;


	if(!(fd=x11amp_connect_to_session(session))) 
		return;
	remote_send_packet(fd,CMD_GET_INFO,NULL,0);
	data = remote_read_packet(fd,&pkt_hdr);
	if(data)
	{
		*rate = ((guint32 *)data)[0];
		*freq = ((guint32 *)data)[1];
		*nch = ((guint32 *)data)[2];
		g_free(data);
	}
	remote_read_ack(fd);
	close(fd);
}

void x11amp_remote_get_eq_data(gint session)
{
	/* write me! */
}

void x11amp_remote_set_eq_data(gint session)
{
	/* write me! */
}

void x11amp_remote_pl_win_toggle(gint session,gboolean show)
{
	remote_send_boolean(session,CMD_PL_WIN_TOGGLE,show);
}

void x11amp_remote_eq_win_toggle(gint session,gboolean show)
{
	remote_send_boolean(session,CMD_EQ_WIN_TOGGLE,show);
}

void x11amp_remote_main_win_toggle(gint session,gboolean show)
{
	remote_send_boolean(session,CMD_MAIN_WIN_TOGGLE,show);
}

void x11amp_remote_show_prefs_box(gint session)
{
	remote_cmd(session,CMD_SHOW_PREFS_BOX);
}

void x11amp_remote_toggle_aot(gint session,gboolean ontop)
{
	remote_send_boolean(session,CMD_TOGGLE_AOT,ontop);
}

void x11amp_remote_show_about_box(gint session)
{
	remote_cmd(session,CMD_SHOW_ABOUT_BOX);
}

void x11amp_remote_eject(gint session)
{
	remote_cmd(session,CMD_EJECT);
}

void x11amp_remote_playlist_prev(gint session)
{
	remote_cmd(session,CMD_PLAYLIST_PREV);
}

void x11amp_remote_playlist_next(gint session)
{
	remote_cmd(session,CMD_PLAYLIST_NEXT);
}

gboolean x11amp_remote_is_running(gint session)
{
	return remote_cmd(session,CMD_PING);
}

void x11amp_remote_toggle_repeat(gint session)
{
	remote_cmd(session,CMD_TOGGLE_REPEAT);
}

void x11amp_remote_toggle_shuffle(gint session)
{
	remote_cmd(session,CMD_TOGGLE_SHUFFLE);
}
